function T = BIAS_CORRECT_MIN(x, Sigma, nlevel, ndraws)

% This function uses simultaneous multi-level bootstrap (Qian, 2011)
% to correct the finite sample bias of min(.) 
% Inputs:
% x            Realization of a MVN random vector
% Sigma   Covariance matrix of MVN, or a vector of standard errors (independence)
% nlevel     number of levels of bootstrap
% ndraws   number of draws to evaluate the analog expectation E(.)
% Output:

% T    Bootstrap corrected estimators of each level
%      T(1) raw estimator (min of the sample)
%      T(2) bootstrap corrected estimator
%      T(3) double bootstraps corrected estimator
%      T(4), T(5),... higher level bootstrap corrected estimator


if nargin < 3;      nlevel = 5;               end
if nargin < 4;      ndraws = 10000;      end


x = x(:);
nvar = length(x);
[nrow,ncol] = size(Sigma);
if nrow * ncol == nvar
    cov_flag = 0;
    Sigma = Sigma(:);
elseif nrow * ncol == nvar^2
    cov_flag = 1;
    chol_P = chol(Sigma);
else
     error('Error: Dimensions of x and Sigma do not match.')
end


% X is hierachical simulated data, 
% each column is a simulation level,
% the first column is the data themselves
X = zeros(nvar, nlevel+1);
X(:,1) = x;

% Bias_mat records the simulated bias
% each column is one level of difference
% each row is one simulated value
Bias_mat = zeros(ndraws, nlevel);

%---------------------------------
% --- Simulation begin ---
%---------------------------------

for r = 1:ndraws
    
    % Generate hierachical simulated data
    if cov_flag == 1
        for m = 2:nlevel+1
            X(:, m) = X(:, m-1) + chol_P' * randn(nvar,1);        
        end
    else
        for m = 2:nlevel+1
            X(:, m) = X(:, m-1) + Sigma .* randn(nvar,1);        
        end
    end
    
    % Compute simulated Bi(x), i = 1,2,3,... by diff-in-diff
    diff_temp = diff(min(X));
    Bias_mat(r,1) = diff_temp(1);
    for m = 2:nlevel
        diff_temp = diff_temp(1:end-1) - diff_temp(2:end);
        Bias_mat(r,m) = diff_temp(1);
    end
end

% Compute B1(x), B2(x), B3(x), stack in a row called Bias
Bias = mean(Bias_mat);

% Compute T1(x), T2(x), T3(x), stack in a row called T
cumsum_Bias = cumsum([0,Bias]);
T = min(x) - cumsum_Bias;


    
    
    
