clear all
tic

% This routine uses the simulation setting in Chernozhukov et.al (2009) 
% Simultaneous multi-level bootstraps (Qian, 2011) added to further correct bias
% T1 raw estimator (max of the sample)
% T2 bootstrap corrected estimator
% T3 double bootstraps corrected estimator
% T4, T5,... higher level bootstrap corrected estimator
% This rountine calls the function Manski_example

myseed = 20110402;
try
    RandStream.setDefaultStream(RandStream('mt19937ar','seed',myseed));
catch
    randn('seed',myseed);
    rand('seed',myseed);
end

% Config parameters
nobs = 500;
nsim = 1000;
nlevel = 4;
ndraws = 10000;
var_flag = 1;

% Simulation study
estimator = zeros(nsim, nlevel+1);
conserve = zeros(nsim,1);
H_status_bar = waitbar(0,'Simulation in Progress   0%');
percentage = floor(nsim / 100);
for r = 1:nsim
    
    if mod(r,percentage) == 0
        waitbar(r / nsim,H_status_bar,['Simulation in Progress   ',num2str(floor(r / nsim * 100)),'%']);
    end
    
    % Z ~ Unif(-2,1.5)
    % D = I[ psi(Z) + N(0,1) > 0 ]
    % Y = 2 psi(Z) + |Z|* TN(0,1)
    % psi(Z) = Z * I(Z<=1) + 1 * I(Z>1)
    Z = rand(nobs,1) * 3.5 - 2;
    psi = Z;
    psi(psi>1) = 1;
    D = (psi + randn(nobs,1)) > 0;
    TN = randn(nobs,1);
    TN(TN>1.96) = 1.96;
    TN(TN<-1.96) = -1.96;
    Y = 2 * psi + abs(Z) .* TN;
    
    % Discretize MIV
    Z = round(Z * 10) / 10;
    M = length(Z);
    
   
    if var_flag ~= 2
        Q = Y;
        Q(D==0) = -1.96;
        fL = zeros(1,M);
        fL_std = zeros(1,M);
        for m = 1:M
            ind = abs(Z - m/M) < 1e-8;
            fL(m) = mean(Q(ind));        
       
            prob_z = mean(ind);
            r_vec = 1:nobs;
            inv_size = sum(1./r_vec .* binopdf(r_vec,nobs,prob_z)) /(1-(1-prob_z)^nobs);
            %disp(['E(size) = ',num2str(1/inv_size),' Realized size = ',num2str(sum(ind))])
            fL_std(m) = sqrt( inv_size * var(Q(ind)) );
        end
    else
        ngrid = 1000;
        [fL, temp1, fL_std, temp2] = MIV_std(Y, D, Z, 1,ngrid);
    end

    estimator(r,:) = BIAS_CORRECT_MAX(fL, fL_std, nlevel, ndraws);
    conserve(r) = max(fL) - mean(max(randn(M,ndraws) .* repmat(fL_std',1,ndraws)));
end
close(H_status_bar);

disp(' ')
theory = 2 * normcdf(1) - (-2 + 2*1.96) * normcdf(-1);
bias = mean(estimator) - theory;
bias_std = std(estimator - theory) / sqrt(nsim);
bias_conserve = mean(conserve) - theory;
bias_conserve_std = std(conserve - theory) / sqrt(nsim);
disp(' ')
for m = 1:nlevel+1
    fprintf( 'Bias T%1.0f = %5.3f \n', m, bias(m));
end
fprintf( 'Bias Tc = %5.3f \n', bias_conserve);



toc



