function [bias,bias_conserve,bias_std,bias_conserve_std] = Example_Manski(M, nobs, sigma2, nsim, nlevel,ndraws, var_flag)

% This function replicate the simulation study in Manski and Pepper(2009) 
% Simultaneous multi-level bootstrap (Qian, 2011) are aslo added
% to further correct the bias.
% Inputs:
% M            MIV has M discrete points
% nobs       Sample size
% sigma2  Variance of simulated yi
% nsim      Number of repetitions in simulation
% nlevel    Number of levels in bootstrap
% ndraws Number of draws for each bootstrap simulation
% var_flag Method to compute variance ( 1 for finite sample, 2 for asymptotic) 
% Output:
% bias      Simulated bias (given M, N, Sigma2) in each level
%               of bootstrap bias correction. The 1st, 2nd 
%               correspond to Table 1,2 in Manski and Pepper(2009)
%      bias(1) bias of the raw estimator (max of the sample)
%      bias(2) bias of the bootstrap corrected estimator
%      bias(3) bias of the double bootstraps corrected estimator
%      bias(4), bias(5),... bias of higher level bootstrap corrected estimator
% bias_conserve Simulated bias of the conservative estimator 
% bias_std Numerical standard error of average bias (std / sqrt(nsim))
% bias_conserve_std Numerical standard error of average bias_conserve (std / sqrt(nsim))

 
if nargin  < 1;     M = 4;              end
if nargin  < 2;     nobs= 1000;     end
if nargin  < 3;     sigma2 = 1;     end
if nargin  < 4;     nsim = 1000;    end
if nargin  < 5;     nlevel = 4;         end
if nargin  < 6;     ndraws = 50000;         end
if nargin  < 7;     var_flag = 1;      end
    

estimator = zeros(nsim, nlevel+1);
conserve = zeros(nsim,1);
H_status_bar = waitbar(0,'Simulation in Progress   0%');
percentage = floor(nsim / 100);
for r = 1:nsim
    
    if mod(r,percentage) == 0
        waitbar(r / nsim,H_status_bar,['Simulation in Progress   ',num2str(floor(r / nsim * 100)),'%']);
    end
    
    %Z = ceil(rand(nobs,1)*M) / M;
    Z = randi(M,[nobs,1]) / M;
    if length(unique(Z)) < M
        disp('MIV Estimator does not exist. Take another draw.')
        Z = randi(M,[nobs,1]) / M;
    end
    D = (Z + randn(nobs,1) ) > 0;
    Y = randn(nobs,1) * sqrt(sigma2);
    Y(Y>1.96) = 1.96;
    Y(Y<-1.96) = -1.96;
    
    if var_flag ~= 2
        Q = Y;
        Q(D==0) = -1.96;
        fL = zeros(1,M);
        fL_std = zeros(1,M);
        for m = 1:M
            ind = abs(Z - m/M) < 1e-8;
            fL(m) = mean(Q(ind));        
       
            prob_z = mean(ind);
            r_vec = 1:nobs;
            inv_size = sum(1./r_vec .* binopdf(r_vec,nobs,prob_z)) /(1-(1-prob_z)^nobs);
            %disp(['E(size) = ',num2str(1/inv_size),' Realized size = ',num2str(sum(ind))])
            fL_std(m) = sqrt( inv_size * var(Q(ind)) );
        end
    else
        ngrid = 1000;
        [fL, temp1, fL_std, temp2] = MIV_std(Y, D, Z, 1,ngrid);
    end

    estimator(r,:) = BIAS_CORRECT_MAX(fL, fL_std, nlevel, ndraws);
    conserve(r) = max(fL) - mean(max(randn(M,ndraws) .* repmat(fL_std',1,ndraws)));
end
close(H_status_bar);

disp(' ')
fprintf('n = %5.0f, M = %3.0f, ^2 = %3.0f,  \n', nobs, M, sigma2);
theory = -1.96 * normcdf(-1);
bias = mean(estimator) - theory;
bias_std = std(estimator - theory) / sqrt(nsim);
bias_conserve = mean(conserve) - theory;
bias_conserve_std = std(conserve - theory) / sqrt(nsim);
disp(' ')
for m = 1:nlevel+1
    fprintf( 'Bias T%1.0f = %5.3f \n', m, bias(m));
end
fprintf( 'Bias Tc = %5.3f \n', bias_conserve);
    


    