function [lb, ub, std_lb,std_ub] = MIV_std(Y, D, Z, D_val,ngrid_max)

% This function calculate the asymptotic standard deviation
% of the treatment reponse (Manski and Pepper,2000)
% under each MIV value
% All variables will be discretized.

% Inputs:
% Y:         data of response variables (column vector)
% D:        data of treatment variable (column vector)
% Z:         data of MIV (column vector)
% D_val: conditional on which treatment?
%             (which latent response, Y1 or Y0)
% ngrid_max: maximum discretized grids

% Outputs
% lb: analog estimator of lower bounds (for max part) under each MIV cell
% lb: analog estimator of upper bounds (for min part) under each MIV cell
% std_lb: the std for lower bounds (for max part) under each MIV cell
% std_ub: the std for upper bounds (for min part) under each MIV cell

if nargin < 3;  error('Error: Data of (Y,D,Z) incomplete.');    end
if nargin < 4;  D_val = max(D);    end
if nargin < 5;  ngrid_max = 50;    end

% Discretize Y, D, Z, and sort from small to large
nobs = length(Y);
Y = double(Y);
D = double(D);
Z = double(Z);

Y_min = min(Y);
Y_max = max(Y);

Y_sort= sort(Y);
D_sort= sort(D);
Z_sort = sort(Z);
for m = 2:nobs
    if Y_sort(m) == Y_sort(m-1)
        Y_sort(m-1) = NaN;
    end
    if D_sort(m) == D_sort(m-1)
        D_sort(m-1) = NaN;
    end
    if Z_sort(m) == Z_sort(m-1)
        Z_sort(m-1) = NaN;
    end
end
Y_sort = Y_sort(~isnan(Y_sort));
D_sort = D_sort(~isnan(D_sort));
Z_sort = Z_sort(~isnan(Z_sort));

if length(Y_sort) > ngrid_max
    Y_sort = linspace(min(Y), max(Y), ngrid_max)';
end
if length(D_sort) > ngrid_max
    D_sort = linspace(min(D), max(D), ngrid_max)';
end
if length(Z_sort) > ngrid_max
    Z_sort = linspace(min(Z), max(Z), ngrid_max)';
end


D_flag = find(D_sort == D_val);
if isnan(D_flag)
    error('Error: The designated D_val is not a value in D')
end

ngrid_Y = length(Y_sort);
ngrid_D = length(D_sort);
ngrid_Z = length(Z_sort);

% Count the frequency of each (Y,D,Z) combination
freq = zeros(ngrid_Y, ngrid_D,ngrid_Z);
for m = 1:nobs
    Y_ind = find(Y(m) <= Y_sort, 1);
    D_ind = find(D(m) <= D_sort, 1);
    Z_ind = find(Z(m) <= Z_sort, 1);
    freq(Y_ind, D_ind, Z_ind) = freq(Y_ind, D_ind, Z_ind) + 1;
end
freq = freq / nobs;


% Calculate the marginal probability of Z
Pz = zeros(ngrid_Y * ngrid_D, ngrid_Z);
Pz(:) = freq(:);
Pz = sum(Pz);
Pz1 = Pz .^ (-1);
Pz2 = Pz .^ (-2);

% Calculate the analog estimator under each MIV value
lb = zeros(1,ngrid_Z);
ub = zeros(1,ngrid_Z);
for Z_ind = 1: ngrid_Z
    temp1 = freq(:, D_flag, Z_ind);
    temp2 = freq(:, [1:D_flag-1;D_flag+1:ngrid_D],Z_ind);
    lb(Z_ind) = temp1(:)' / Pz(Z_ind) * Y_sort +  Y_min * sum(temp2(:)) / Pz(Z_ind);
    ub(Z_ind) = temp1(:)' / Pz(Z_ind) * Y_sort +  Y_max * sum(temp2(:)) / Pz(Z_ind);
end

% Calculate the std for each MIV value by Delta Method
std_lb = zeros(1, ngrid_Z);
std_ub = zeros(1, ngrid_Z);
for Z_ind = 1:ngrid_Z    
    prob_not_D = freq(:, [1:D_flag-1;D_flag+1:ngrid_D],Z_ind); 
    prob_not_D = sum(prob_not_D(:));
    
    % To compute the gradient, we loop element by element    
    Grad_lb = zeros(ngrid_Y, ngrid_D);
    Grad_ub = zeros(ngrid_Y, ngrid_D);
    for Y_ind = 1:ngrid_Y
        for D_ind = 1:ngrid_D
            
            % involve a summation, another loop
            Grad = 0;
            for s = 1:ngrid_Y
                if s == Y_ind && D_ind == D_flag
                    Grad = Grad + Y_sort(s) * (Pz1(Z_ind) - freq(s, D_ind,Z_ind) * Pz2(Z_ind));
                else
                    Grad = Grad  + Y_sort(s) * ( - freq(s, D_ind,Z_ind) * Pz2(Z_ind));
                end
            end
            
            if D_ind ~= D_flag                
                Grad_lb(Y_ind,D_ind) = Grad + Y_min * (Pz1(Z_ind) - prob_not_D * Pz2(Z_ind));
                Grad_ub(Y_ind,D_ind) = Grad + Y_max * (Pz1(Z_ind) - prob_not_D * Pz2(Z_ind));
            else
                Grad_lb(Y_ind,D_ind) = Grad + Y_min * (prob_not_D * Pz2(Z_ind));
                Grad_ub(Y_ind,D_ind) = Grad + Y_max * (prob_not_D * Pz2(Z_ind));
            end
                        
        end
    end
    
    freq_diag = freq(:,:,Z_ind);
    freq_diag = freq_diag(:);
    std_lb(Z_ind) = sqrt(sum(freq_diag .* Grad_lb(:) .^ 2) / nobs);
    std_ub(Z_ind) = sqrt(sum(freq_diag .* Grad_ub(:) .^ 2) / nobs);
end
